@@ -252,7 +252,8 @@ void ScreenShotSinkPipe2::flush(const QImage &image)
         const QByteArray buffer(reinterpret_cast<const char *>(image.constBits()),
                                 image.sizeInBytes());
         writeBufferToPipe(fileDescriptor, buffer);
-    }, m_fileDescriptor, image);
+    },
+                      m_fileDescriptor, image);
 
     // The ownership of the pipe file descriptor has been moved to the worker thread.
     m_fileDescriptor = -1;
@@ -285,6 +286,11 @@ bool ScreenShotDBusInterface2::checkPermissions() const
         return false;
     }
 
+    static bool permissionCheckDisabled = qEnvironmentVariableIntValue("KWIN_SCREENSHOT_NO_PERMISSION_CHECKS") == 1;
+    if (permissionCheckDisabled) {
+        return true;
+    }
+
     const QDBusReply<uint> reply = connection().interface()->servicePid(message().service());
     if (reply.isValid()) {
         const uint pid = reply.value();
@@ -334,7 +340,7 @@ QVariantMap ScreenShotDBusInterface2::CaptureWindow(const QString &handle,
         return QVariantMap();
     }
 
-    EffectWindow *window = effects->findWindow(handle);
+    EffectWindow *window = effects->findWindow(QUuid(handle));
     if (!window) {
         bool ok;
         const int winId = handle.toInt(&ok);
@@ -455,7 +461,7 @@ QVariantMap ScreenShotDBusInterface2::CaptureInteractive(uint kind,
     const QDBusMessage replyMessage = message();
 
     if (kind == 0) {
-        effects->startInteractiveWindowSelection([=](EffectWindow *window) {
+        effects->startInteractiveWindowSelection([=, this](EffectWindow *window) {
             effects->hideOnScreenMessage(EffectsHandler::OnScreenMessageHideFlag::SkipsCloseAnimation);
 
             if (!window) {
@@ -472,7 +478,7 @@ QVariantMap ScreenShotDBusInterface2::CaptureInteractive(uint kind,
                                           "Escape or right click to cancel."),
                                      QStringLiteral("spectacle"));
     } else {
-        effects->startInteractivePositionSelection([=](const QPoint &point) {
+        effects->startInteractivePositionSelection([=, this](const QPoint &point) {
             effects->hideOnScreenMessage(EffectsHandler::OnScreenMessageHideFlag::SkipsCloseAnimation);
 
             if (point == QPoint(-1, -1)) {
